#include <csl.h>
#include <csl_irq.h>
#include <csl_edma.h>
#include <math.h> 

#include "sobel.h"
#include "memport.h"
#include "../common/commdef.h"

#include <stdlib.h>


//#define EMIFB_CECTL2 ((unsigned *)0x01A80010)
//#define EMIFB_CESEC2 ((unsigned *)0x01A80050)

//debug
#define EMIFA_GBLCTL ((unsigned *)0x01800000)
#define EMIFA_CE0CTL ((unsigned *)0x01800008)
#define EMIFA_CE0SEC ((unsigned *)0x01800048)
#define EMIFA_SDCTL ((unsigned *)0x01800018)
#define EMIFA_SDTIM ((unsigned *)0x0180001C)
#define EMIFA_SDEXT ((unsigned *)0x01800020)

volatile Int32 timer_done;
volatile unsigned read_done;
volatile unsigned ext_int;
volatile Uint32 *imageptrout; 
volatile Uint32 wr_num_sec;
volatile Uint32 rd_num_sec;
int main()
{
	Int32 i,len, imagelen, mod_len;
	Int32 data_out_sec;
	Int32 data_in_sec;
	Int32 mask;
  volatile Int32 *imageptrin, *imagebuff;
  Int32 i_part;
  Uint32 secaddress;
  
  
  //SDRAM interface initialization  
  // low-lev init  
  *EMIFA_GBLCTL = 0x000000a0;
  *EMIFA_CE0CTL = 0x000000d0;
  *EMIFA_SDTIM = 0x00000300;
  *EMIFA_SDEXT = 0x00054528;
  *EMIFA_SDCTL = 0x53116000;

  

  
  
	//Set input image address
	hpicAccess(SOBEL_INPUT) = IMAGE_START;
	//Set output image address
	hpicAccess(SOBEL_OUTPUT) = OUTPUT_START;
	hpicAccess(SOBEL_DONE) = 0;

	//Initialization
	CSL_init();
	//Init memport library
	MemportInit();

	for(;;){
		hpicAccess(SOBEL_DATA_IN_READY) = 0;
    //Wait while input data not ready
		while (hpicAccess(SOBEL_DATA_IN_READY) == 0) hpicAccess(SOBEL_DONE) = 1; 
		
		hpicAccess(SOBEL_DONE) = 0;
		hpicAccess(SOBEL_DATA_OUT_READY) = 0;
		hpicAccess(SOBEL_DATA_OUT_LEN) = 0;

    //Input image pointer
    imageptrin = (Int32 *)IMAGE_START;
    //Output image pointer
    imageptrout = (Uint32 *)OUTPUT_START;
    
    //Input image data length
		imagelen = hpicAccess(SOBEL_DATA_IN_LEN);

    //Process data
    for(i_part=0; i_part<imagelen/(NUM_READ_SEC * MEMPORT_SECTOR_SIZE); i_part++){
      //length of data to be loaded to input memoryports
      len = (NUM_READ_SEC * MEMPORT_SECTOR_SIZE);
      
      //
      //--INPUT DATA-----------------------------------------------------------------------------------------------------------
      //
      
      //set memoryports address to 0
  		(*MEMPORT_ADDR)=0;
  		
  		//load data
  		
  		read_done=0;
  		EDMA_qdmaConfigArgs(
  			0x41180001,
  			EDMA_SRC_OF(imageptrin),
  			EDMA_CNT_OF(len/4),
  			EDMA_DST_OF(MEMPORT_DATA),
  			EDMA_IDX_OF(0x00000004)
  		);
  		imageptrin+=len/4;
  		while(read_done==0);
      		
  		//number of used memoryports
  		rd_num_sec = len/MEMPORT_SECTOR_SIZE;
  		if((len%MEMPORT_SECTOR_SIZE)!=0) rd_num_sec++;
  
      //saturation
  		if (rd_num_sec > NUM_READ_SEC) rd_num_sec = NUM_READ_SEC;

      //make ready port mask
  		data_in_sec = 0;
  		mask=1;
  		for(i=0; i<rd_num_sec; i++){
  			data_in_sec |= mask;
  			mask <<= 1;
  		}
  		data_in_sec |= 0xFFF0; 
      wr_num_sec = 0;
      read_done = 0;
      //set memory port ready
  		(*MEMPORT_READY) = data_in_sec;
      
      //
      //--OUTPUT DATA----------------------------------------------------------------------------------------------------------
      //

  		//wait while input data are pocessed
  		//while(((*MEMPORT_READY) & 0x000f) !=0xf);
  		//Output sectors are cleared by interrupt rutine and all input sectors are ready
   		while(((*MEMPORT_READY)) !=0xf);
    }//end for

    if(imagelen%(NUM_READ_SEC * MEMPORT_SECTOR_SIZE)!=0){
      //length of data to be loaded to input memoryports
      mod_len = imagelen%(NUM_READ_SEC * MEMPORT_SECTOR_SIZE);
      
      //
      //--INPUT DATA-----------------------------------------------------------------------------------------------------------
      //
      
      //set memoryports address to 0
  		(*MEMPORT_ADDR)=0;
  		
  		//load data
  		
  		read_done=0;
  		EDMA_qdmaConfigArgs(
  			0x41180001,
  			EDMA_SRC_OF(imageptrin),
  			EDMA_CNT_OF(mod_len/4),
  			EDMA_DST_OF(MEMPORT_DATA),
  			EDMA_IDX_OF(0x00000004)
  		);
  		imageptrin+=mod_len/4;
  		while(read_done==0);
      		
  		//number of used memoryports
  		rd_num_sec = mod_len/MEMPORT_SECTOR_SIZE;
  		if((mod_len%MEMPORT_SECTOR_SIZE)!=0) rd_num_sec++;
  
      //saturation
  		if (rd_num_sec > NUM_READ_SEC) rd_num_sec = NUM_READ_SEC;

      //make ready port mask
  		data_in_sec = 0;
  		mask=1;
  		for(i=0; i<rd_num_sec; i++){
  			data_in_sec |= mask;
  			mask <<= 1;
  		}
  		data_in_sec |= 0xFFF0; 
      wr_num_sec = 0;
      read_done = 0;
      //set memory port ready
  		(*MEMPORT_READY) = data_in_sec;
      
      //
      //--OUTPUT DATA----------------------------------------------------------------------------------------------------------
      //

  		//wait while input data are pocessed
  		//while(((*MEMPORT_READY) & 0x000f) !=0xf);
  		//Output sectors are cleared by interrupt rutine and all input sectors are ready
   		while(((*MEMPORT_READY)) !=0xf);
    }
    
    //Output data length
		hpicAccess(SOBEL_DATA_OUT_LEN) = (i_part * len * 2) + (mod_len * 2);
		//Output data ready
		hpicAccess(SOBEL_DATA_OUT_READY) = 1;
	}
}

