-------------------------------------------------------------------------------------
-- Entity:	DX64_cicb
--
-- Project:	Uni1p
--
-- Author:	Antonin Hegar
--
-- Description:
--	Top entity of XC2V250 on DX64 v1.0
--
-- Revision:
--	$Log: DX64_cicb.vhd,v $
--	Revision 1.1  2006/05/05 17:05:50  Tony
--	Initial version.
--	Entity to test CiCb.
--	
--
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------
-- LIBRARY
-------------------------------------------------------------------------------------
library ieee;
library unisim;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.DX64Header.all;
use work.MathLib.all;
use unisim.vcomponents.all;

-------------------------------------------------------------------------------------
-- entity
-------------------------------------------------------------------------------------
entity dx64_sobel is
	generic (
		DEBUG_C 		: boolean := true
	);
	port (
		-----------------------------------------------------------------------------
		-- global signals
		NReset			: inout std_logic;
		Clock			: in std_logic;

        -----------------------------------------------------------------------------
		-- DSP interface
		NDReset			: out std_logic;
		NMI				: out std_logic;
		BEA				: in std_logic_vector(8 downto 1);
		BED				: inout std_logic_vector(15 downto 0);
		--BBE				: inout std_logic_vector(1 downto 0);
		BCE				: in std_logic_vector(3 downto 0);
		BARE			: in std_logic;
		BAWE			: in std_logic;
		BAOE			: in std_logic;
		BSO3			: in std_logic;
		BARDY			: out std_logic;
		BPDT			: in std_logic;
		BECLKOUT		: in std_logic;

		BECLKIN			: out std_logic;
		AECLKIN			: out std_logic;        
		ClkMode			: out std_logic_vector(1 downto 0);

 		ExtInt4			: out std_logic;
 		ExtInt5			: out std_logic;
 		ExtInt6			: out std_logic;        

        -----------------------------------------------------------------------------
		-- General input/output to master FPGA
		--GIO				: inout std_logic_vector(7 downto 0);

		LGreen			: out std_logic;
		LYellow			: out std_logic;
		LRed			: out std_logic
	);
end entity dx64_sobel;


----------------------------------------------------------------
-- ENTITY
-------------------------------------------------------------------------------------
architecture struct of dx64_sobel is

	---------------------------------------------------------------------------------
	-- Components declaration
	---------------------------------------------------------------------------------
  
  
  
  
  
    --
  -- added by insertbm
  --
  component bm_inst_wrapper_0 is
  port(
    rst_in : in std_logic;
    rst_out : out std_logic;
    data_in_in : in std_logic_vector(15 downto 0);
    data_in_out : out std_logic_vector(15 downto 0);
    valid_sector_in_in : in std_logic_vector(3 downto 0);
    valid_sector_in_out : out std_logic_vector(3 downto 0);
    clear_sector_out_in : in std_logic_vector(7 downto 0);
    clear_sector_out_out : out std_logic_vector(7 downto 0);
    mem_in_addr_in : in std_logic_vector(10 downto 0);
    mem_in_addr_out : out std_logic_vector(10 downto 0);
    mem_in_addr_en : in std_logic_vector(10 downto 0);
    clear_sector_in_in : in std_logic_vector(3 downto 0);
    clear_sector_in_out : out std_logic_vector(3 downto 0);
    clear_sector_in_en : in std_logic_vector(3 downto 0);
    data_out_in : in std_logic_vector(49 downto 0);
    data_out_out : out std_logic_vector(49 downto 0);
    data_out_en : in std_logic_vector(49 downto 0);
    data_out_valid_in : in std_logic;
    data_out_valid_out : out std_logic;
    data_out_valid_en : in std_logic;
    mem_out_addr_in : in std_logic_vector(10 downto 0);
    mem_out_addr_out : out std_logic_vector(10 downto 0);
    mem_out_addr_en : in std_logic_vector(10 downto 0);
    valid_sector_out_in : in std_logic_vector(7 downto 0);
    valid_sector_out_out : out std_logic_vector(7 downto 0);
    valid_sector_out_en : in std_logic_vector(7 downto 0)
  );
  end component bm_inst_wrapper_0;

  
  
  
  
	attribute box_type		: string;
	-- Digital Clock Manager
	component DCM is
	generic (
   	    CLKIN_PERIOD 		: real := 0.0;
    	DFS_FREQUENCY_MODE 	: string := "LOW";
    	DLL_FREQUENCY_MODE 	: string := "LOW";
    	DSS_MODE 			: string := "NONE";
    	DUTY_CYCLE_CORRECTION : boolean := true;
    	PHASE_SHIFT 		: integer := 0;
    	CLKDV_DIVIDE 		: real := 2.0;
		CLKIN_DIVIDE_BY_2 	: boolean := false;
    	CLKOUT_PHASE_SHIFT 	: string := "NONE";
	   	CLKFX_DIVIDE 		: integer := 1;
    	CLKFX_MULTIPLY 		: integer := 4;
    	CLK_FEEDBACK 		: string := "1X";
    	STARTUP_WAIT 		: boolean := false
    );
	port (
	    CLK0 		: out std_ulogic := '0';
	    CLK180 		: out std_ulogic := '0';
	    CLK270 		: out std_ulogic := '0';
	    CLK2X 		: out std_ulogic := '0';
	    CLK2X180 	: out std_ulogic := '0';
	    CLK90 		: out std_ulogic := '0';
	    CLKDV 		: out std_ulogic := '0';
	    CLKFX 		: out std_ulogic := '0';
	    CLKFX180 	: out std_ulogic := '0';
	    LOCKED 		: out std_ulogic := '0';
	    PSDONE 		: out std_ulogic := '0';
	    STATUS 		: out std_logic_vector(7 downto 0) := "00000000";

	    CLKFB 		: in std_ulogic := '0';
	    CLKIN 		: in std_ulogic := '0';
	    DSSEN 		: in std_ulogic := '0';
	    PSCLK 		: in std_ulogic := '0';
	    PSEN 		: in std_ulogic := '0';
	    PSINCDEC 	: in std_ulogic := '0';
	    RST 		: in std_ulogic := '0'
 	);
	end component DCM;
	attribute box_type of DCM : component is "black_box";

	---------------------------------------------------------------------------------
	-- IBUFG
	component IBUFG
	port(
		O : out std_ulogic;
		I : in std_ulogic);
	end component;
	attribute box_type of IBUFG : component is "black_box";

	---------------------------------------------------------------------------------
	-- BUFG
	component BUFG
	port(
		O : out std_ulogic;
		I : in std_ulogic);
	end component;
   	attribute box_type of BUFG : component is "black_box";


--
-- added by insertIOB
--
component IBUF
  port (
    I : in std_logic;
    O : out std_logic
  );
end component IBUF;

 attribute box_type of IBUF: component is "primitive";

component OBUF
  port (
    I : in std_logic;
    O : out std_logic
  );
end component OBUF;
attribute box_type of OBUF: component is "primitive";


component IOBUF
  port (
    O : out std_logic;
    IO : inout std_logic;
    I : in std_logic;
    T : in std_logic
  );
end component IOBUF;
attribute box_type of IOBUF: component is "primitive";


  component BUFGP is
    port (
      I : in std_logic;
      O : out std_logic
    );
  end component;
  attribute box_type of BUFGP : component is "primitive";


	---------------------------------------------------------------------------------
	-- EMIFB interface
	component EmifB is
	generic (
        CLKIN_FREQ_C    : real := 50.0e6;
		EMIF_FREQ_C		: real := 50.0e6;
		N_DEVICES_C		: natural := 2;
		DCM_MULTIPLY_C	: natural := 2;
		DCM_DIVIDE_C	: natural := 1
	);
	port (
		nreset_in			: in std_logic;                     	-- external asynchronous reset
		nreset_out			: out std_logic;                     	-- external asynchronous reset
		nreset_t			: out std_logic;                     	-- external asynchronous reset		
    ClockExt		: in std_logic;
      Debug : out std_logic_vector(7 downto 0);
		NDReset			: out std_logic;
		NMI				: out std_logic;
		BEA				: in std_logic_vector(8 downto 1);
		--BED				: inout std_logic_vector(15 downto 0);
		BED_in				: in std_logic_vector(15 downto 0);
		BED_out				: out std_logic_vector(15 downto 0);
		BED_tri				: out std_logic_vector(15 downto 0);
    		--BBE				: inout std_logic_vector(1 downto 0);
		BCE				: in std_logic_vector(3 downto 0);
		BARE			: in std_logic;
		BAWE			: in std_logic;
		BAOE			: in std_logic;
		BSO3			: in std_logic;
		BARDY			: out std_logic;
		BPDT			: in std_logic;
		BECLKIN			: out std_logic;
		BECLKOUT		: in std_logic;
		AReset			: out std_logic;
		ClockInt		: out std_logic;
		ClockFx			: out std_logic;
		DspBusArr		: inout DspBusArr_T(0 to N_DEVICES_C-1)
	);
	end component EmifB;

      
      -------------------------------------------------------------------------
      -- Application MemoryPort
      -------------------------------------------------------------------------
      component MemoryPort
          generic (
              SECTOR_FIRST_C  : natural;
              SECTOR_LAST_C   : natural;
              DATA_WIDTH_C    : natural;
              ADDR_WIDTH_C    : natural;
              BLOCK_SUPPORT_C : boolean);
          port (
              AReset          : in    std_logic;
              Clock           : in    std_logic;
              Debug           : out   unsigned(31 downto 0);
              DspBus          : inout DspBus_T;
              ChainNext       : in    MemoryPortChain_T;
              ChainPrev       : out   MemoryPortChain_T;
              BramAddr        : in    unsigned(ADDR_WIDTH_C-1 downto 0);
              BramDataIn      : in    unsigned(DATA_WIDTH_C-1 downto 0);
              BramDataOut     : out   unsigned(DATA_WIDTH_C-1 downto 0);
              BramWrite       : in    std_logic;
              DspSectorValid  : out   unsigned(SECTOR_LAST_C downto SECTOR_FIRST_C);
              DspSectorClear  : in    unsigned(SECTOR_LAST_C downto SECTOR_FIRST_C);
              FpgaSectorValid : in    unsigned(SECTOR_LAST_C downto SECTOR_FIRST_C);
              FpgaSectorClear : out   unsigned(SECTOR_LAST_C downto SECTOR_FIRST_C);
              FpgaSectorOvf   : in    unsigned(SECTOR_LAST_C downto SECTOR_FIRST_C));
      end component;

      -------------------------------------------------------------------------
      -- Memory port to transfer data to the DSP form FPGA's application
      -------------------------------------------------------------------------
      component MemoryPortOut
          generic (
              SECTOR_FIRST_C : natural;
              SECTOR_LAST_C  : natural;
              DATA_WIDTH_C   : natural;
              ADDR_WIDTH_C   : natural); 
          port (
              AReset          : in    std_logic;
              Clock           : in    std_logic;
              DspBus          : inout DspBus_T;
              ChainNext       : in    MemoryPortChain_T;
              ChainPrev       : out   MemoryPortChain_T;
				  BramAddr 			: in 	  unsigned(ADDR_WIDTH_C-2 downto 0);
				  BramDataIn 		: in 	  unsigned((2 * DATA_WIDTH_C)-1 downto 0);
              BramWrite       : in    std_logic;
              FpgaSectorValid : in    unsigned(SECTOR_LAST_C downto SECTOR_FIRST_C);
              FpgaSectorClear : out   unsigned(SECTOR_LAST_C downto SECTOR_FIRST_C);
              FpgaSectorOvf   : in    unsigned(SECTOR_LAST_C downto SECTOR_FIRST_C)); 
      end component;

      -------------------------------------------------------------------------
      -- Memory port to transfer data from the DSP to the FPGA's application
      -------------------------------------------------------------------------
      component MemoryPortIn
          generic (
              SECTOR_FIRST_C : natural;
              SECTOR_LAST_C  : natural;
              DATA_WIDTH_C   : natural;
              ADDR_WIDTH_C   : natural); 
          port (
              AReset         : in    std_logic;
              Clock          : in    std_logic;
              DspBus         : inout DspBus_T;
              ChainNext      : in    MemoryPortChain_T;
              ChainPrev      : out   MemoryPortChain_T;
              BramAddr       : in    unsigned(ADDR_WIDTH_C-1 downto 0);
              BramDataOut    : out   unsigned(DATA_WIDTH_C-1 downto 0);
              DspSectorValid : out   unsigned(SECTOR_LAST_C downto SECTOR_FIRST_C);
              DspSectorClear : in    unsigned(SECTOR_LAST_C downto SECTOR_FIRST_C)); 
      end component;
      
      -------------------------------------------------------------------------
      -- MemoryPort testing component
      -------------------------------------------------------------------------
      component MemoryPortTest
          generic (
              WR_SECTORS_C    : natural;
              RD_SECTORS_C    : natural;
              WR_ADDR_WIDTH_C : natural;
              RD_ADDR_WIDTH_C : natural);
          port (
              AReset        : in  std_logic;
              Clock         : in  std_logic;
              Debug         : out unsigned(31 downto 0);
              RdSectorValid : in  unsigned(RD_SECTORS_C-1 downto 0);
              RdSectorClear : out unsigned(RD_SECTORS_C-1 downto 0);
              RdBramAddr    : out unsigned(RD_ADDR_WIDTH_C-1 downto 0);
              RdBramDataIn  : in  unsigned(DSP_DATA_WIDTH_C-1 downto 0);
              WrSectorValid : out unsigned(WR_SECTORS_C-1 downto 0);
              WrSectorOvf   : out unsigned(WR_SECTORS_C-1 downto 0);
              WrSectorClear : in  unsigned(WR_SECTORS_C-1 downto 0);
              WrBramAddr    : out unsigned(WR_ADDR_WIDTH_C-1 downto 0);
              WrBramDataOut : out unsigned(DSP_DATA_WIDTH_C-1 downto 0);
              WrBramWrite   : out std_logic);
      end component;

	
    ---------------------------------------------------------------------------
    -- Timer
    ---------------------------------------------------------------------------  
      component Timer
          generic (
              BASE_ADDR_C  : natural;
              FREE_RUN_C   : boolean;
              TIMER_FREQ_C : natural);
          port (
              AReset : in    std_logic;
              Clock  : in    std_logic;
              DspBus : inout DspBus_T;
              DspInt : out   std_logic);
      end component;



-- Definition of wrapper interface
  component wrapper is
	 generic (
		 C_DATA_IN_WIDTH  : integer := 16;
		 C_DATA_OUT_WIDTH  : integer := 50;
		 C_LATENCY : integer := 2;
		 C_RD_SEC : integer := 4;
		 C_WR_SEC : integer := 4;
		 MEM_IN_ADDR_WD : integer := 10;
		 MEM_OUT_ADDR_WD : integer := 9;
		 MEM_IN_DATA_WD : integer := 16;
		 MEM_OUT_DATA_WD : integer := 32
	 );
   port( 
		clk : in  std_logic;
      rst : in  std_logic;

      data_in : in unsigned(C_DATA_IN_WIDTH-1 downto 0);
      mem_in_addr : out unsigned(MEM_IN_ADDR_WD-1 downto 0);      
      valid_sector_in : in unsigned(C_RD_SEC-1 downto 0); 
      clear_sector_in : out unsigned(C_RD_SEC-1 downto 0); 

      data_out : out unsigned(C_DATA_OUT_WIDTH-1 downto 0);
      data_out_valid : out std_logic;
      mem_out_addr : out unsigned(MEM_OUT_ADDR_WD-1 downto 0);
      valid_sector_out : out unsigned(C_WR_SEC-1 downto 0);
      clear_sector_out : in unsigned(C_WR_SEC-1 downto 0)
	);
  end component;

      
	---------------------------------------------------------------------------------
	-- Type definition and constants
	---------------------------------------------------------------------------------
      
	---------------------------------------------------------------------------------
	-- Signal definition
	---------------------------------------------------------------------------------
	
	
--
-- added by insertIOB
--
signal nreset_ibuf : std_logic;
signal bea_ibuf : std_logic_vector(8 downto 1);
signal bce_ibuf : std_logic_vector(3 downto 0);
signal bare_ibuf : std_logic;
signal bawe_ibuf : std_logic;
signal baoe_ibuf : std_logic;
signal bso3_ibuf : std_logic;
signal bpdt_ibuf : std_logic;
signal ndreset_obuf : std_logic;
signal nmi_obuf : std_logic;
signal bardy_obuf : std_logic;
signal beclkin_obuf : std_logic;
signal aeclkin_obuf : std_logic;
signal clkmode_obuf : std_logic_vector(1 downto 0);
signal extint4_obuf : std_logic;
signal extint5_obuf : std_logic;
signal extint6_obuf : std_logic;
signal lgreen_obuf : std_logic;
signal lyellow_obuf : std_logic;
signal lred_obuf : std_logic;
signal clock_BUFGP : std_logic;
signal beclkout_BUFGP : std_logic;
	
	
	
	
  -- added by insertbm
  --
  signal rst_i_bm_inst_wrapper_0_s2m : std_logic;
  signal data_in_i_bm_inst_wrapper_0_s2m : std_logic_vector(15 downto 0);
  signal valid_sector_in_i_bm_inst_wrapper_0_s2m : std_logic_vector(3 downto 0);
  signal clear_sector_out_i_bm_inst_wrapper_0_s2m : std_logic_vector(7 downto 0);
  signal mem_in_addr_i_bm_inst_wrapper_0_m2s : std_logic_vector(10 downto 0);
  signal clear_sector_in_i_bm_inst_wrapper_0_m2s : std_logic_vector(3 downto 0);
  signal data_out_i_bm_inst_wrapper_0_m2s : std_logic_vector(49 downto 0);
  signal data_out_valid_i_bm_inst_wrapper_0_m2s : std_logic;
  signal mem_out_addr_i_bm_inst_wrapper_0_m2s : std_logic_vector(10 downto 0);
  signal valid_sector_out_i_bm_inst_wrapper_0_m2s : std_logic_vector(7 downto 0);
  signal i_bm_inst_wrapper_0_enable : std_logic_vector(49 downto 0);
	
  signal uns_data_in_i_bm_inst_wrapper_0_s2m : unsigned(15 downto 0);
  signal uns_valid_sector_in_i_bm_inst_wrapper_0_s2m : unsigned(3 downto 0);
  signal uns_clear_sector_out_i_bm_inst_wrapper_0_s2m : unsigned(7 downto 0);
  signal uns_mem_in_addr_i_bm_inst_wrapper_0_m2s : unsigned(10 downto 0);
  signal uns_clear_sector_in_i_bm_inst_wrapper_0_m2s : unsigned(3 downto 0);
  signal uns_data_out_i_bm_inst_wrapper_0_m2s : unsigned(49 downto 0);
  signal uns_mem_out_addr_i_bm_inst_wrapper_0_m2s : unsigned(10 downto 0);
  signal uns_valid_sector_out_i_bm_inst_wrapper_0_m2s : unsigned(7 downto 0);
  signal uns_i_bm_inst_wrapper_0_enable : unsigned(49 downto 0);
	
	signal slv_rd_bram_dataout : std_logic_vector(15 downto 0);
	signal slv_rd_bram_addr : std_logic_vector(10 downto 0);
	signal slv_rd_dsp_valid : std_logic_vector(3 downto 0);
	signal slv_wr_fpga_clear : std_logic_vector(7 downto 0);
	signal slv_rd_dsp_clear : std_logic_vector(3 downto 0);
	signal slv_sobel_data_out : std_logic_vector(49 downto 0);
	signal slv_wr_bram_addr : std_logic_vector(10 downto 0);
	signal slv_wr_fpga_valid : std_logic_vector(7 downto 0);
   
   
   
   signal nreset_in : std_logic;
   signal nreset_out : std_logic;
   signal nreset_t : std_logic;
   
   signal bed_in : std_logic_vector(15 downto 0);
   signal bed_out : std_logic_vector(15 downto 0);
   signal bed_tri : std_logic_vector(15 downto 0);
	
    signal sreset_in        : std_logic := '1';
    signal sreset_in2       : std_logic := '1';
	signal areset_in		: std_logic;
	signal emif_reset		: std_logic;
	signal clock_in			: std_logic;
	signal clock_fx			: std_logic;
	signal dcm2_clk2x		: std_logic;
	signal dcm2_clk2x180	: std_logic;
	signal dcm_locked_tx	: std_logic;
	signal dcm_reset_tx		: std_logic;
	signal dcm2_psdone		: std_logic;
	signal tx_ser_clock		: std_logic;
	signal tx_ser_clock180	: std_logic;
    signal beclkin_in       : std_logic;
      
	signal lgreen_in		: std_logic;
	constant BLINK_CNT_MAX_C: natural := 50e6/2;
	signal blink_cnt		: natural range 0 to BLINK_CNT_MAX_C;


    ---------------------------------------------------------------------------
    -- Local bus
    ---------------------------------------------------------------------------
	constant N_DEVICES_C	: natural := 4;
	signal dsp_bus_arr	: DspBusArr_T(0 to N_DEVICES_C-1);


    ---------------------------------------------------------------------------
    -- Receiver application: FPGA -> DSP
    ---------------------------------------------------------------------------
    -- constants 
    constant WR_SECTORS_C    : natural := 8;
    constant WR_ADDR_WIDTH_C : natural := log2(MEMPORT_SECTOR_SIZE_C * WR_SECTORS_C -1);

    -- memory port interface
    signal wr_bram_addr    : unsigned(WR_ADDR_WIDTH_C-2 downto 0);
    signal wr_bram_datain  : unsigned((2 * DSP_DATA_WIDTH_C)-1 downto 0);
    signal wr_bram_write   : std_logic;
    signal wr_fpga_valid   : unsigned(WR_SECTORS_C-1 downto 0);
    signal wr_fpga_clear   : unsigned(WR_SECTORS_C-1 downto 0);
    signal wr_fpga_ovf     : unsigned(WR_SECTORS_C-1 downto 0);

    ---------------------------------------------------------------------------
    -- Transmitter application: DSP -> FPGA 
    ---------------------------------------------------------------------------
    -- constants 
    constant RD_SECTORS_C    : natural := 4;
    constant RD_ADDR_WIDTH_C : natural := log2(MEMPORT_SECTOR_SIZE_C * RD_SECTORS_C -1);

    -- memory port interface
    signal rd_bram_addr    : unsigned(RD_ADDR_WIDTH_C-1 downto 0);
    signal rd_bram_dataout : unsigned(DSP_DATA_WIDTH_C-1 downto 0);
    signal rd_bram_write   : std_logic;
    signal rd_dsp_valid    : unsigned(RD_SECTORS_C-1 downto 0);
    signal rd_dsp_clear    : unsigned(RD_SECTORS_C-1 downto 0);


    signal chain0           : MemoryPortChain_T;
    signal chain1           : MemoryPortChain_T;      
    signal mem_debug        : unsigned(31 downto 0);
    signal port0_debug      : unsigned(31 downto 0);
      
   signal nmi_in			: std_logic;
	signal ndreset_in		: std_logic;
	signal ext_int4_out	    : std_logic;
	signal ext_int5_out	    : std_logic;      
	signal ext_int6_out	    : std_logic;
   signal timer_int      : std_logic;

    ---------------------------------------------------------------------------
	
	
	signal memport_debug	: unsigned(31 downto 0);      
	signal emif_debug		: std_logic_vector(7 downto 0);
	signal timer_debug		: std_logic_vector(15 downto 0);
   signal cicbmem_debug    : unsigned(31 downto 0);      

	signal nrst_int        : std_logic;
	signal nrst_int2       : std_logic;
	signal nreset_internal : std_logic;
	
	
	signal sobel_data_out : unsigned(49 downto 0);
      
begin

	---------------------------------------------------------------------------------
	-- CLOCK GENERATION
	---------------------------------------------------------------------------------
	-- DCM
	DCM2_G: DCM
	generic map (
    	DFS_FREQUENCY_MODE 	=> "LOW",
	    DLL_FREQUENCY_MODE 	=> "LOW",
		CLKIN_DIVIDE_BY_2 	=> false,
    	CLKOUT_PHASE_SHIFT 	=> "NONE",
	   	CLKFX_DIVIDE 		=> 1,
    	CLKFX_MULTIPLY 		=> 2,
    	CLK_FEEDBACK 		=> "2X"
    )
	port map (
	    CLK0 		=> OPEN,
	    CLK180 		=> OPEN,
	    CLK270 		=> OPEN,
	    CLK2X 		=> dcm2_clk2x,
	    CLK2X180 	=> dcm2_clk2x180,
	    CLK90 		=> OPEN,
	    CLKDV 		=> OPEN,
	    CLKFX 		=> OPEN,
	    CLKFX180 	=> OPEN,
	    LOCKED 		=> dcm_locked_tx,
	    PSDONE 		=> dcm2_psdone,
	    STATUS 		=> OPEN,

	    CLKFB 		=> tx_ser_clock,
	    CLKIN 		=> clock_fx,
	    DSSEN 		=> '0',
	    PSCLK 		=> '0',
	    PSEN 		=> '0',
	    PSINCDEC 	=> '0',
	    RST 		=> dcm_reset_tx
 	);

	------------------------------------------------------------------------------------------
	-- output clock buffers
	------------------------------------------------------------------------------------------
	ClkTx_G: BUFG
			port map (I => dcm2_clk2x, O => tx_ser_clock);

	ClkTx180_G: BUFG
			port map (I => dcm2_clk2x180, O => tx_ser_clock180);


	---------------------------------------------------------------------------------
	-- EMIFB interface
	EmifB_G: EmifB
	generic map (
        CLKIN_FREQ_C    => real(INPUT_CLOCK_FREQ_C),
        EMIF_FREQ_C		=> real(FPGA_CLOCK_FREQ_C),
		N_DEVICES_C		=> N_DEVICES_C,
		DCM_MULTIPLY_C	=> 2,
		DCM_DIVIDE_C	=> 1
	)
	port map (
		nreset_in			=> nreset_in,                     	-- external asynchronous reset
		nreset_out			=> nreset_out,                    	-- external asynchronous reset
		nreset_t			=> nreset_t,                	-- external asynchronous reset
		
		
		ClockExt		=> clock_BUFGP,
	Debug => emif_debug,
		NDReset			=> ndreset_in,
		NMI				=> nmi_in,
		BEA				=> bea_ibuf,
		BED_in				=> BED_in,
		BED_out				=> BED_out,
		BED_tri				=> BED_tri,		
    --BBE				=> BBE,
		BCE				=> bce_ibuf,
		BARE			=> bare_ibuf,
		BAWE			=> bawe_ibuf,
		BAOE			=> baoe_ibuf,
		BSO3			=> bso3_ibuf,
		BARDY			=> bardy_obuf,
		BPDT			=> bpdt_ibuf,
		BECLKIN			=> beclkin_in,
		BECLKOUT		=> beclkout_BUFGP,
		AReset			=> emif_reset,
		ClockInt		=> clock_in,
		ClockFx			=> clock_fx,
		DspBusArr		=> dsp_bus_arr
	);

--process (NReset, Clock)
--begin
--	if Clock'event and Clock = '1' then
--		nrst_int   <= NReset;
--		nrst_int2 <= nrst_int;
--	end if;
--end process;

--   nreset_internal <= nrst_int; 

      beclkin_obuf <= beclkin_in;
      aeclkin_obuf <= beclkin_in;
      
    ---------------------------------------------------------------------------------
	-- APPLICATION0 - TIMER ---------------------------------------------------------
	---------------------------------------------------------------------------------
	Timer_G: Timer
        generic map (
            BASE_ADDR_C => 32,
            FREE_RUN_C  => TRUE,
            TIMER_FREQ_C => 1e6)
        port map (
            AReset		=> areset_in,
            Clock		=> clock_in,
            DspInt      => timer_int, 
            DspBus		=> dsp_bus_arr(0)
	);


    ---------------------------------------------------------------------------------
	-- APPLICATION1 - RECEIVER ------------------------------------------------------
	---------------------------------------------------------------------------------
	-- Memory port to write by application in the fpga (FPGA -> DSP)
    MemoryPortWrite_G: MemoryPortOut
          generic map (
              SECTOR_FIRST_C => RD_SECTORS_C,
              SECTOR_LAST_C  => RD_SECTORS_C+WR_SECTORS_C-1,
              DATA_WIDTH_C   => DSP_DATA_WIDTH_C,
              ADDR_WIDTH_C   => WR_ADDR_WIDTH_C)
          port map (
              AReset          => areset_in,
              Clock           => clock_in,
              DspBus          => dsp_bus_arr(1),
              ChainNext       => chain1,
              ChainPrev       => chain0,
              BramAddr        => wr_bram_addr,
              BramDataIn      => wr_bram_datain,
              BramWrite       => wr_bram_write,
              FpgaSectorValid => wr_fpga_valid,
              FpgaSectorClear => wr_fpga_clear,
              FpgaSectorOvf   => wr_fpga_ovf);

	-- Memory port to write by the DSP (DSP -> FPGA)
      MemoryPortIn_G: MemoryPortIn
          generic map (
              SECTOR_FIRST_C  => 0,
              SECTOR_LAST_C   => RD_SECTORS_C-1,
              DATA_WIDTH_C    => DSP_DATA_WIDTH_C,
              ADDR_WIDTH_C    => RD_ADDR_WIDTH_C)
          port map (
              AReset         => areset_in,
              Clock          => clock_in,
              DspBus         => dsp_bus_arr(2),
              ChainNext      => LAST_MEM_PORT_C,
              ChainPrev      => chain1,
              BramAddr       => rd_bram_addr,
              BramDataOut    => rd_bram_dataout,
              DspSectorValid => rd_dsp_valid,
              DspSectorClear => rd_dsp_clear);

    	

	inst_wrapper: wrapper 
	 generic map(
		 C_DATA_IN_WIDTH  => DSP_DATA_WIDTH_C,
		 C_DATA_OUT_WIDTH  => 50,
		 C_LATENCY => 2,
		 C_RD_SEC => RD_SECTORS_C,
		 C_WR_SEC => WR_SECTORS_C,
		 MEM_IN_ADDR_WD => RD_ADDR_WIDTH_C,
		 MEM_OUT_ADDR_WD =>WR_ADDR_WIDTH_C-1,
		 MEM_IN_DATA_WD => 16,
		 MEM_OUT_DATA_WD => 32
	 )
   port map( 
-- 		clk => clock_in,
--       rst => areset_in,
-- 
--       data_in  => rd_bram_dataout,
--       mem_in_addr => rd_bram_addr,
--       valid_sector_in => rd_dsp_valid,
--       clear_sector_in => rd_dsp_clear,
-- 
--       data_out => sobel_data_out,--wr_bram_datain,
--       data_out_valid => wr_bram_write,
--       mem_out_addr => wr_bram_addr,
--       valid_sector_out => wr_fpga_valid, 
--       clear_sector_out => wr_fpga_clear
      
      
      
      
      
		  clk => clock_in,
      rst => rst_i_bm_inst_wrapper_0_s2m,

      data_in  => uns_data_in_i_bm_inst_wrapper_0_s2m,
      mem_in_addr => uns_mem_in_addr_i_bm_inst_wrapper_0_m2s,
      valid_sector_in => uns_valid_sector_in_i_bm_inst_wrapper_0_s2m,
      clear_sector_in => uns_clear_sector_in_i_bm_inst_wrapper_0_m2s,

      data_out => uns_data_out_i_bm_inst_wrapper_0_m2s,
      data_out_valid => data_out_valid_i_bm_inst_wrapper_0_m2s,
      mem_out_addr => uns_mem_out_addr_i_bm_inst_wrapper_0_m2s,
      valid_sector_out => uns_valid_sector_out_i_bm_inst_wrapper_0_m2s,
      clear_sector_out => uns_clear_sector_out_i_bm_inst_wrapper_0_s2m      
      
      
      
	);	


  uns_data_in_i_bm_inst_wrapper_0_s2m <= unsigned(data_in_i_bm_inst_wrapper_0_s2m);
  slv_rd_bram_dataout <= std_logic_vector(rd_bram_dataout);

  mem_in_addr_i_bm_inst_wrapper_0_m2s <= std_logic_vector(uns_mem_in_addr_i_bm_inst_wrapper_0_m2s);
  rd_bram_addr <= unsigned(slv_rd_bram_addr);

  uns_valid_sector_in_i_bm_inst_wrapper_0_s2m <= unsigned(valid_sector_in_i_bm_inst_wrapper_0_s2m);
  slv_rd_dsp_valid <= std_logic_vector(rd_dsp_valid);
  
  clear_sector_in_i_bm_inst_wrapper_0_m2s <= std_logic_vector(uns_clear_sector_in_i_bm_inst_wrapper_0_m2s);
  rd_dsp_clear <= unsigned(slv_rd_dsp_clear);
  
  
  data_out_i_bm_inst_wrapper_0_m2s <= std_logic_vector(uns_data_out_i_bm_inst_wrapper_0_m2s);
  sobel_data_out <= unsigned(slv_sobel_data_out);
  
  mem_out_addr_i_bm_inst_wrapper_0_m2s <= std_logic_vector(uns_mem_out_addr_i_bm_inst_wrapper_0_m2s);
  wr_bram_addr <= unsigned(slv_wr_bram_addr);
  
  valid_sector_out_i_bm_inst_wrapper_0_m2s <= std_logic_vector(uns_valid_sector_out_i_bm_inst_wrapper_0_m2s);
  
  uns_clear_sector_out_i_bm_inst_wrapper_0_s2m <= unsigned(clear_sector_out_i_bm_inst_wrapper_0_s2m);
  slv_wr_fpga_clear <= std_logic_vector(wr_fpga_clear);
  
  
  
  
--
-- added by insertbm
--
i_bm_inst_wrapper_0_enable <= (others => '1');


	--
	-- added by insertbm
	--
i_bm_inst_wrapper_0 : bm_inst_wrapper_0
  port map (
    rst_in => areset_in,
    rst_out => rst_i_bm_inst_wrapper_0_s2m,
    data_in_in => slv_rd_bram_dataout,
    data_in_out => data_in_i_bm_inst_wrapper_0_s2m,
    valid_sector_in_in => slv_rd_dsp_valid,
    valid_sector_in_out => valid_sector_in_i_bm_inst_wrapper_0_s2m,
    clear_sector_out_in => slv_wr_fpga_clear,
    clear_sector_out_out => clear_sector_out_i_bm_inst_wrapper_0_s2m,
    mem_in_addr_in => mem_in_addr_i_bm_inst_wrapper_0_m2s,
    mem_in_addr_out => slv_rd_bram_addr,
    mem_in_addr_en => i_bm_inst_wrapper_0_enable(10 downto 0),
    clear_sector_in_in => clear_sector_in_i_bm_inst_wrapper_0_m2s,
    clear_sector_in_out => slv_rd_dsp_clear,
    clear_sector_in_en => i_bm_inst_wrapper_0_enable(3 downto 0),
    data_out_in => data_out_i_bm_inst_wrapper_0_m2s,
    data_out_out => slv_sobel_data_out,
    data_out_en => i_bm_inst_wrapper_0_enable(49 downto 0),
    data_out_valid_in => data_out_valid_i_bm_inst_wrapper_0_m2s,
    data_out_valid_out => wr_bram_write,
    data_out_valid_en => i_bm_inst_wrapper_0_enable(0),
    mem_out_addr_in => mem_out_addr_i_bm_inst_wrapper_0_m2s,
    mem_out_addr_out => slv_wr_bram_addr,
    mem_out_addr_en => i_bm_inst_wrapper_0_enable(10 downto 0),
    valid_sector_out_in => valid_sector_out_i_bm_inst_wrapper_0_m2s,
    valid_sector_out_out => slv_wr_fpga_valid,
    valid_sector_out_en => i_bm_inst_wrapper_0_enable(7 downto 0)
  );



--
-- added by insertIOB
--

 
        IOBUF_NRESET : IOBUF
            port map (
                O  => nreset_in,
                IO => Nreset,
                I  => nreset_out,
                T  => nreset_t
            );




        IOBUF_BED0 : IOBUF
            port map (
                O  => bed_in(0),
                IO => BED(0),
                I  => bed_out(0),
                T  => bed_tri(0)
            );
        IOBUF_BED1 : IOBUF
            port map (
                O  => bed_in(1),
                IO => BED(1),
                I  => bed_out(1),
                T  => bed_tri(1)
            );
        IOBUF_BED2 : IOBUF
            port map (
                O  => bed_in(2),
                IO => BED(2),
                I  => bed_out(2),
                T  => bed_tri(2)
            );
        IOBUF_BED3 : IOBUF
            port map (
                O  => bed_in(3),
                IO => BED(3),
                I  => bed_out(3),
                T  => bed_tri(3)
            );
        IOBUF_BED4 : IOBUF
            port map (
                O  => bed_in(4),
                IO => BED(4),
                I  => bed_out(4),
                T  => bed_tri(4)
            );
        IOBUF_BED5 : IOBUF
            port map (
                O  => bed_in(5),
                IO => BED(5),
                I  => bed_out(5),
                T  => bed_tri(5)
            );
        IOBUF_BED6 : IOBUF
            port map (
                O  => bed_in(6),
                IO => BED(6),
                I  => bed_out(6),
                T  => bed_tri(6)
            );
        IOBUF_BED7 : IOBUF
            port map (
                O  => bed_in(7),
                IO => BED(7),
                I  => bed_out(7),
                T  => bed_tri(7)
            );
        IOBUF_BED8 : IOBUF
            port map (
                O  => bed_in(8),
                IO => BED(8),
                I  => bed_out(8),
                T  => bed_tri(8)
            );
        IOBUF_BED9 : IOBUF
            port map (
                O  => bed_in(9),
                IO => BED(9),
                I  => bed_out(9),
                T  => bed_tri(9)
            );
        IOBUF_BED10 : IOBUF
            port map (
                O  => bed_in(10),
                IO => BED(10),
                I  => bed_out(10),
                T  => bed_tri(10)
            );
        IOBUF_BED11 : IOBUF
            port map (
                O  => bed_in(11),
                IO => BED(11),
                I  => bed_out(11),
                T  => bed_tri(11)
            );
        IOBUF_BED12 : IOBUF
            port map (
                O  => bed_in(12),
                IO => BED(12),
                I  => bed_out(12),
                T  => bed_tri(12)
            );
        IOBUF_BED13 : IOBUF
            port map (
                O  => bed_in(13),
                IO => BED(13),
                I  => bed_out(13),
                T  => bed_tri(13)
            );
        IOBUF_BED14 : IOBUF
            port map (
                O  => bed_in(14),
                IO => BED(14),
                I  => bed_out(14),
                T  => bed_tri(14)
            );
        IOBUF_BED15 : IOBUF
            port map (
                O  => bed_in(15),
                IO => BED(15),
                I  => bed_out(15),
                T  => bed_tri(15)
            );


 
ibuf_1 : IBUF
  port map (
    I => bea(1),
    O => bea_ibuf(1)
  );
 
ibuf_2 : IBUF
  port map (
    I => bea(2),
    O => bea_ibuf(2)
  );
 
ibuf_3 : IBUF
  port map (
    I => bea(3),
    O => bea_ibuf(3)
  );
 
ibuf_4 : IBUF
  port map (
    I => bea(4),
    O => bea_ibuf(4)
  );
 
ibuf_5 : IBUF
  port map (
    I => bea(5),
    O => bea_ibuf(5)
  );
 
ibuf_6 : IBUF
  port map (
    I => bea(6),
    O => bea_ibuf(6)
  );
 
ibuf_7 : IBUF
  port map (
    I => bea(7),
    O => bea_ibuf(7)
  );
 
ibuf_8 : IBUF
  port map (
    I => bea(8),
    O => bea_ibuf(8)
  );
 
ibuf_9 : IBUF
  port map (
    I => bce(0),
    O => bce_ibuf(0)
  );
 
ibuf_10 : IBUF
  port map (
    I => bce(1),
    O => bce_ibuf(1)
  );
 
ibuf_11 : IBUF
  port map (
    I => bce(2),
    O => bce_ibuf(2)
  );
 
ibuf_12 : IBUF
  port map (
    I => bce(3),
    O => bce_ibuf(3)
  );
 
ibuf_13 : IBUF
  port map (
    I => bare,
    O => bare_ibuf
  );
 
ibuf_14 : IBUF
  port map (
    I => bawe,
    O => bawe_ibuf
  );
 
ibuf_15 : IBUF
  port map (
    I => baoe,
    O => baoe_ibuf
  );
 
ibuf_16 : IBUF
  port map (
    I => bso3,
    O => bso3_ibuf
  );
 
ibuf_17 : IBUF
  port map (
    I => bpdt,
    O => bpdt_ibuf
  );
 
obuf_0 : OBUF
  port map (
    I => ndreset_obuf,
    O => ndreset
  );
 
obuf_1 : OBUF
  port map (
    I => nmi_obuf,
    O => nmi
  );
 
obuf_2 : OBUF
  port map (
    I => bardy_obuf,
    O => bardy
  );
 
obuf_3 : OBUF
  port map (
    I => beclkin_obuf,
    O => beclkin
  );

obuf_3a : OBUF
  port map (
    I => aeclkin_obuf,
    O => aeclkin
  );
 
obuf_4 : OBUF
  port map (
    I => clkmode_obuf(0),
    O => clkmode(0)
  );
 
obuf_5 : OBUF
  port map (
    I => clkmode_obuf(1),
    O => clkmode(1)
  );
 
obuf_6 : OBUF
  port map (
    I => extint4_obuf,
    O => extint4
  );

obuf_6e5 : OBUF
  port map (
    I => extint5_obuf,
    O => extint5
  );

obuf_6e6 : OBUF
  port map (
    I => extint6_obuf,
    O => extint6
  );
 
obuf_7 : OBUF
  port map (
    I => lgreen_obuf,
    O => lgreen
  );
 
obuf_8 : OBUF
  port map (
    I => lyellow_obuf,
    O => lyellow
  );
 
obuf_9 : OBUF
  port map (
    I => lred_obuf,
    O => lred
  );
  
  
  bufgp_clk_ext : BUFGP
    port map (
      I => clock,
      O => clock_BUFGP
    );

  bufgp_beclkout : BUFGP
    port map (
      I => beclkout,
      O => beclkout_BUFGP
    );
    
----------------------








    ---------------------------------------------------------------------------
    -- sync reset
    ---------------------------------------------------------------------------
    SReset_P: process (areset_in, clock_in)
    begin
        if areset_in = '1' then
            sreset_in2  <= '1';
            sreset_in   <= '1';
        elsif clock_in'event and clock_in = '1' then
            sreset_in2  <= areset_in;
            sreset_in   <= sreset_in2;
        end if;
    end process SReset_P;
            
      
    ---------------------------------------------------------------------------
    -- DSP interrupts
    ---------------------------------------------------------------------------
	extint4_obuf <= chain0.SectorReady;
    extint5_obuf <= chain0.SectorOvf;
    extint6_obuf <= timer_int;  

    ---------------------------------------------------------------------------
    -- DSP control signals
    ---------------------------------------------------------------------------
	clkmode_obuf	<= "01";
	nmi_obuf 	<= nmi_in;
	ndreset_obuf <= ndreset_in;

    ---------------------------------------------------------------------------
    -- Internal reset
    ---------------------------------------------------------------------------
	areset_in 		<= emif_reset OR (NOT dcm_locked_tx);
	dcm_reset_tx	<= emif_reset;

	--GIO <= (others => 'Z');

	---------------------------------------------------------------------------
	-- TEST BLINKING 
	---------------------------------------------------------------------------	
	Blink_P: process (areset_in, clock_in)
	begin
		if areset_in = '1' then
			blink_cnt 	<= 0;
			lgreen_in	<= '1';
		elsif clock_in'event AND clock_in = '1' then
			if blink_cnt = BLINK_CNT_MAX_C then
				blink_cnt <= 0;
				lgreen_in <= NOT lgreen_in;
			else
				blink_cnt <= blink_cnt + 1;
			end if;
		end if;
	end process Blink_P;

	-- LEDs
	lgreen_obuf 	<= lgreen_in;
	lyellow_obuf <= areset_in;
	lred_obuf	<= '0' when ext_int4_out = '1' or ext_int5_out = '1' else '1';


	wr_bram_datain <= sobel_data_out(47 downto 16);--sobel_data_out(31 downto 0);

end struct;
